# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['lapidary_base']

package_data = \
{'': ['*']}

install_requires = \
['httpx[http2]>=0.23.0,<0.24.0',
 'pydantic>=1.10.2,<2.0.0',
 'python-mimeparse>=1.6.0,<2.0.0']

setup_kwargs = {
    'name': 'lapidary-base',
    'version': '0.5.0',
    'description': 'Base classes for clients generated by Lapidary project',
    'long_description': 'Base classes for [Lapidary](https://github.com/pytohon-lapidary/lapidary) async OpenAPI client generator.\n',
    'author': 'Raphael Krupinski',
    'author_email': 'rafalkrupinski@users.noreply.github.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/python-lapidary/lapidary-base',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
